package service

import (
	"context"
	service "doc-service/domain/repository"
	"doc-service/infra/db"
	"encoding/json"
	"github.com/nacos-group/nacos-sdk-go/v2/common/logger"
	pb "gitlab.galaxy-immi.com/Backend-group/proto/pb/docservice"
	"strings"
	"time"
)

type DocServiceService struct {
	pb.UnimplementedDocServiceServiceServer
}

func recoverPanic() {
	if r := recover(); r != nil {
		logger.Error("异常了", r)
	}
}

func inFileType(fileType string) bool {

	if fileType == "" {
		return false
	}

	// 示例数组
	sliceType := []string{
		"doc", "docx", "wps", "wpss", "docm", "dotm", "dot", "dotx", "html",
		"pptx", "ppt", "pot", "potx", "pps", "ppsx", "dps", "dpt", "pptm", "potm", "ppsm", "dpss",
		"xls", "xlt", "et", "ett", "xlsx", "xltx", "csv", "xlsb", "xlsm", "xltm", "ets",
	}

	for _, v := range sliceType {
		if v == fileType {
			return true
		}
	}
	return false

}

/*
**
发起文件转换
格式：(source_type)
文字文档（Word）：doc","docx","wps","wpss","docm","dotm","dot","dotx","html。
演示文档（PPT）：pptx","ppt","pot","potx","pps","ppsx","dps","dpt","pptm","potm","ppsm","dpss。
表格文档（Excel）：xls","xlt","et","ett","xlsx","xltx","csv","xlsb","xlsm","xltm","ets。

输出格式:(target_type)
图片：png","jpg。
文本：txt。
PDF：pdf。
*/
func (doc *DocServiceService) DoConversion(c context.Context, req *pb.DoConversionRequest) (resp *pb.DoConversionResponse, err error) {
	defer recoverPanic()
	resp = &pb.DoConversionResponse{}
	CallBackData := &pb.CallBackData{
		Id: req.CallbackData.Id,
	}
	resp.Code = 10001
	resp.Data = nil
	if req.SourceUrl == "" {
		resp.Msg = "SourceUrl 不能为空"
		return resp, err
	}
	if req.ProjectName == "" {
		resp.Msg = "ProjectName 不能为空"
		return resp, err
	}
	if req.SourceType == "" {
		resp.Msg = "SourceType 不能为空"
		return resp, err
	}
	if req.CallbackUrl == "" {
		resp.Msg = "CallbackUrl 不能为空"
		return resp, err
	}
	//校验格式
	if !inFileType(req.SourceType) {
		return resp, err
	}

	if req.TargetType == "" {
		req.TargetType = "pdf"
	}

	if req.TargetUrl == "" {
		dotIndex := strings.LastIndex(req.SourceUrl, ".")
		if dotIndex != -1 {
			// 从点的位置开始，替换后面的所有字符为指定字符串
			req.TargetUrl = req.SourceUrl[:dotIndex+1] + "pdf"
		} else {
			resp.Msg = "SourceUrl 不合法"
			return resp, err
		}
	}

	var isPrivate bool = true
	if req.IsPrivate != nil {
		isPrivate = *req.IsPrivate
	}

	jsonCallBackData, _ := json.Marshal(CallBackData)
	request := &service.DoConversionRequest{
		ProjectName:  req.ProjectName,
		SourceType:   req.SourceType,
		SourceURI:    req.SourceUrl,
		TargetType:   req.TargetType,
		TargetURI:    req.TargetUrl,
		IsPrivate:    isPrivate,
		CallbackData: string(jsonCallBackData),
		CallbackUrl:  req.CallbackUrl,
	}

	//创建项目
	createErr := service.NewConversion().CreateProject(req.ProjectName)
	if createErr != nil {
		resp.Msg = createErr.Error()
		resp.Code = 10001
		resp.Data = nil
		return resp, err
	}

	//入库 默认公有有库
	isPri := 1
	if !isPrivate {
		isPri = 0
	}
	save, errsInfo := db.GetServerSiteMicrosDB().DocServerProject.Create().
		SetProjectName(req.ProjectName).
		SetCreatedAt(time.Now()).
		SetSourceURL(request.SourceURI).
		SetSourceType(req.SourceType).
		SetTargetType(req.TargetType).
		SetIsPrivate(int8(isPri)).
		SetTargetURL(request.TargetURI).
		SetCallbackRes("").
		Save(c)

	if errsInfo != nil {
		resp.Msg = errsInfo.Error()
		resp.Code = 10001
		resp.Data = nil
		return resp, errsInfo
	}

	//添加回调
	back, errBack := db.GetServerSiteMicrosDB().DocServerCallback.Create().
		SetCreatedAt(time.Now()).
		SetCallbackData(request.CallbackData).
		SetCallbackURL(request.CallbackUrl).
		SetStatus(0).
		SetServerProjectID(save.ID).
		Save(c)

	if errBack != nil {
		resp.Msg = errBack.Error()
		resp.Code = 10001
		resp.Data = nil
		return resp, errBack
	}

	resp.Msg = "success"
	resp.Code = 200
	resp.Data = &pb.DoConversionData{
		Id:         save.ID,
		TaskId:     "",
		CallbackId: back.ID,
	}
	return resp, nil
}

// 获取转换结果

func (doc *DocServiceService) GetProjectInfo(c context.Context, req *pb.GetProjectRequest) (resp *pb.GetProjectResponse, err error) {
	defer recoverPanic()
	resp = &pb.GetProjectResponse{}
	ConversionInfo := &pb.DoConversionInfo{}

	request := &service.GetTaskRequest{
		ProjectId:   req.ProjectId,
		ProjectName: req.ProjectName,
	}
	request.RequestDefinition = true

	if request.ProjectId <= 0 {
		resp.Msg = "项目id不能为空"
		resp.Code = 10001
		resp.Data = nil
		return resp, err
	}

	projectInfo, infoErr := db.GetServerSiteMicrosDB().DocServerProject.Get(c, req.ProjectId)
	if infoErr != nil {
		resp.Msg = err.Error()
		resp.Code = 10001
		resp.Data = nil
		return resp, err
	}
	request.ProjectName = projectInfo.ProjectName
	request.TaskId = projectInfo.TaskID
	//task_type和原始有区别，需要使用返回的task_id的首个元素
	splitted := strings.Split(projectInfo.TaskID, "-")
	request.TaskType = splitted[0]

	ConversionInfo.ProjectName = projectInfo.ProjectName
	ConversionInfo.Id = projectInfo.ID
	ConversionInfo.TargetUrl = *projectInfo.TargetURL
	ConversionInfo.SourceType = projectInfo.SourceType
	ConversionInfo.TaskId = projectInfo.TaskID
	ConversionInfo.TargetType = projectInfo.TargetType

	result, errs := service.NewConversion().GetTaskProject(*request)
	//获取入库
	CreateGetProject(c, request.ProjectId, *result.Body.Status)
	if errs != nil || *result.StatusCode != 200 || *result.Body.Status != "Succeeded" {
		resp.Msg = err.Error()
		resp.Code = 10001
		resp.Data = nil
		return resp, err
	}

	resp.Msg = *result.Body.Status
	resp.Code = *result.StatusCode
	resp.Data = ConversionInfo
	return resp, nil
}

func CreateGetProject(c context.Context, projectId int32, aliRes string) {
	var statusData int8
	if aliRes != "Succeeded" {
		statusData = 1
	} else {
		statusData = 2
	}
	_, _ = db.GetServerSiteMicrosDB().DocServerGetProject.Create().
		SetCreatedAt(time.Now()).
		SetUpdatedAt(time.Now()).
		SetStatus(statusData).
		SetCurrNum(1).
		SetAliResult(aliRes).
		SetServerProjectID(projectId).
		Save(c)
}
